<!doctype html>
<html lang="en" dir="ltr">
<head>
  <link href="https://tabler.github.io/tabler/assets/css/dashboard.css" rel="stylesheet">
  <style>
    .flex-column {
      display: flex;
      flex-direction: column;
    }
  </style>
</head>
<body>
  <div class="page">
    <div class="flex-column justify-content-center align-items-center">
      <div class="card col-login">
        <div class="card-body flex-column align-items-center bg-info text-white text-center">
          <div class="mb-4">
            <i class="fe fe-mail"></i>
            <i class="fe fe-check"></i>
          </div>
          <h1>Un email vous a été envoyé pour vous connecter.</h1>
          <div class="mb-6">Si vous ne le recevez pas, vous pouvez réessayer.</div>
          <div>
            <a href="{% url 'magicauth-login' %}?next={{ next_url|urlencode }}" class="btn btn-secondary">Réessayer</a>
          </div>
        </div>
      </div>
    </div>
  </div>
</body>
</html>
